@@ -1,10 +1,14 @@
 #####RAT - Research Assessment Tools
-#####Version 0.1.1 (2021-12-04)
+#####Version 0.2.0 (2022-04-05)
 #####By Pedro Cardoso & Stefano Mammola
 #####Maintainer: pedro.cardoso@helsinki.fi
 #####Reference: Cardoso, P., Fukushima, C.S. & Mammola, S. (subm.) Quantifying the international collaboration of researchers and research institutions.
-#####Changed from v0.1.0:
-#####Corrected errors in CRAN
+#####Changed from v0.1.1:
+#####Added function r.index
+#####Added parameters r, h, homeCountry, and logbase to i.index
+#####Added parameter ext to i.map
+#####Added algorithm to standardize names
+#####Added algorithm to identify matches when country name is not recognized
 
 #####required packages
 library("ggplot2")
@@ -22,6 +26,24 @@ library("wosr")
 
 globalVariables(c("map", "world", "region", "x", "y"))
 
+#Function to standardize country names
+stdCountries <- function(countries){
+  for(i in 1:length(countries)){
+
+    #try to find country in map$country
+    if(countries[i] %in% map$country){
+      countries[i] = map[which(map$country == countries[i]), ]$stdCountry
+
+    #if not found identify best match using fuzzy matching (Levenshtein edit distance)
+    } else {
+      d = adist(map$country, countries[i])
+      countries[i] = map$country[which(d == min(d))][1]
+    }
+  }
+
+  return(countries)
+}
+
 ################################################################################
 ################################MAIN FUNCTIONS##################################
 ################################################################################
@@ -65,6 +87,7 @@ wos <- function(id, user = NULL, pass = NULL, coll = c("SCI", "SSCI", "AHCI", "I
 #' @references Hirsch, J.E. (2005). An index to quantify an individual's scientific research output. PNAS, 102: 16569–16572. doi:10.1073/pnas.0507655102.
 #' @examples id = wos("C-2482-2012")
 #' h.index(id)
+#' h.index(id, TRUE)
 #' @export
 h.index <- function(id, fulldata = FALSE){
 
@@ -93,25 +116,43 @@ h.index <- function(id, fulldata = FALSE){
 #' I-index.
 #' @description Calculates the i-index based on Web of Science data.
 #' @param id A list obtained with function 'wos'.
+#' @param r if TRUE the i-index is multiplied by the r-index, i.e., weighted according to the expected distribution of GDP values of collaborating countries.
+#' @param h if TRUE the i-index is divided by the h-index to create a measure independent of the latter.
+#' @param homeCountry A character string specifying the country of origin of the researcher to calculate the r-index if r = TRUE. Look at map$country for the complete list. If NULL, the country with most hits in Web of Science is used.
+#' @param logbase The log base for building the octaves of the r-index if r = TRUE.
 #' @param fulldata if TRUE returns publication and citation counts.
-#' @details The i-index is a measure of scientific collaborations across countries. Calculated as the i number of co-author countries in more than i papers (Cardoso et al. subm.).
+#' @details The i-index (internationalization) is a measure of scientific collaborations across countries. Calculated as the i number of co-author countries in more than i papers (Cardoso et al. subm.).
+#' The weighted version of the index multiplies its raw value by the square rooted difference between observed and expected distribution of GDP per capita of countries constituting the index (function RAT::represent).
+#' The standardized distribution divides the i-index (weighted or not) by the h-index as these two are usually correlated.
 #' @return The i-index value. If fulldata = TRUE a list with full data.
 #' @references Cardoso, P., Fukushima, C.S. & Mammola, S. (subm.) Quantifying the international collaboration attitude of scholars.
 #' @examples id = wos("C-2482-2012")
-#' i.index(id, fulldata = TRUE)
+#' i.index(id)
+#' i.index(id, r = TRUE)
+#' i.index(id, r = TRUE, h = TRUE, logbase = 10, fulldata = TRUE)
 #' @export
-i.index <- function(id, fulldata = FALSE){
+i.index <- function(id, r = FALSE, h = FALSE, homeCountry = NULL, logbase = 2, fulldata = FALSE){
 
   if(is.null(id)) return()
 
   #get country data from each paper
-  countries = unique(id$address[, c('ut', 'country')])
+  countries = id$address[, c('ut', 'country')]
+  countries$country = stdCountries(countries$country)
+  countries = unique(countries)
   countries = table(countries$country)
   countries = countries[order(countries, decreasing = TRUE)]
 
   #calculate i as the i countries in more than i papers
   i = length(which(countries >= 1:length(countries)))
 
+  #weight by GDP distribution
+  if(r)
+    i = i * r.index(id, homeCountry, logbase)
+
+  #standardize by h-index
+  if(h)
+    i = i / h.index(id)
+
   #extra data
   if(fulldata){
     n_countries = length(countries)
@@ -121,10 +162,69 @@ i.index <- function(id, fulldata = FALSE){
   return(i)
 }
 
+#' R-index.
+#' @description Calculates the r-index based on Web of Science data.
+#' @param id A list obtained with function 'wos'.
+#' @param homeCountry A character string specifying the country of origin of the researcher. Look at map$country for the complete list. If NULL, the country with most hits in Web of Science is used.
+#' @param logbase The log base for building the octaves.
+#' @param plot plots the expected and observed distribution of collaborations according to GDP.
+#' @details The r-index (representativeness) is a measure of the overlap between observed and expected distributions of GDP per capita of collaborating countries (Cardoso et al. subm.).
+#' The abundance distribution of log(GDP per capita) of countries in the collaborators list is calculated (using octaves). This is compared with the global distribution of GDPs by using the overlap of both lists.
+#' @return The r-index value.
+#' @references Cardoso, P., Fukushima, C.S. & Mammola, S. (subm.) Quantifying the international collaboration attitude of scholars.
+#' @examples id = wos("C-2482-2012")
+#' r.index(id)
+#' r.index(id, logbase = 10, plot = TRUE)
+#' @export
+r.index <- function(id, homeCountry = NULL, logbase = 2, plot = FALSE){
+
+  if(is.null(id)) return()
+
+  #get country data from each paper and exclude home country
+  countries = id$address[, c('ut', 'country')]
+  countries$country = stdCountries(countries$country)
+  countries = unique(countries)$country
+  if(is.null(homeCountry)){
+    count = table(countries)
+    count = count[order(count, decreasing = TRUE)]
+    homeCountry = names(count)[1] #take the country with most matches
+  }
+  countries = countries[countries != homeCountry]
+
+  #return 0 if no countries beyond own country
+  if (length(countries) == 0)
+    return(0)
+
+  #observed GDP distribution by octaves
+  gdp = c()
+  for(i in 1:length(countries))
+    gdp[i] = as.numeric(map[which(map$stdCountry == countries[i])[1], 3])
+  gdp = as.integer(log(gdp, logbase))
+  gdp = data.frame(table(gdp)/sum(table(gdp)))
+
+  #global GDP distribution by octaves
+  global = as.numeric(unique(map[, 2:3])$gdpPerCapita)
+  global = as.integer(log(global, logbase))
+  global = data.frame(table(global)/sum(table(global)))
+
+  #calculate overlap as the sum of min values
+  colnames(gdp) = colnames(global) = c("octave", "frequence")
+  overlap = merge(global, gdp, by = "octave", all.x = TRUE, all.y = TRUE)
+  overlap[is.na(overlap)] = 0
+
+  if(plot)
+    barplot(t(as.matrix(overlap[, 2:3])), legend.text = c("expected", "observed"), beside = TRUE)
+
+  overlap = sum(apply(overlap[, 2:3], 1, min))
+
+  return(overlap)
+}
+
 #' Map of international collaboration.
 #' @description Generates a network of international collaboration.
 #' @param id A list obtained with function 'wos'.
 #' @param homeCountry A character string specifying the country of origin of the researcher. Look at map$country for the complete list. If NULL, the country with most hits in Web of Science is used.
+#' @param ext extent of the bounding box of the map in decimal degrees (minX, maxX, minY, maxY).
 #' @param sea.col A character indicating the color of the sea.
 #' @param country.col A character indicating the color of the countries in the world.
 #' @param country.border.col A character indicating the color of the border among countries.
@@ -148,6 +248,7 @@ i.index <- function(id, fulldata = FALSE){
 #' i.map(id, country.size.proportional = TRUE)
 #' @export
 i.map <- function(id, homeCountry = NULL,
+                  ext = c(-180, 180, -90, 90),
                   sea.col = "black",
                   country.col = "grey",
                   country.border.col = "black",
@@ -181,18 +282,23 @@ i.map <- function(id, homeCountry = NULL,
     homeCountry = names(count)[1] #take the country with most matches
 
   #add coordinates
-  countries   <- map[map[,1] %in% names(count),]
-  homeCountry <- countries[countries$country %in% homeCountry,]
+  countries <- map[map[,1] %in% names(count), ]
+  homeCountry <- countries[countries$country %in% homeCountry, ]
 
   #Set size for countries
   if(country.size.proportional == TRUE){
     country.point.size = count[sort(names(count))]
     country.point.size = country.point.size[names(country.point.size) %in% countries$country]
-
   }
   if(is.null(homeCountry.point.size))
     homeCountry.point.size = sqrt(max(country.point.size, na.rm = TRUE))
 
+  #convert as.numeric
+  homeCountry$x <- as.numeric(homeCountry$x)
+  homeCountry$y <- as.numeric(homeCountry$y)
+  countries$x   <- as.numeric(countries$x)
+  countries$y   <- as.numeric(countries$y)
+
   #Make the plot
   net <- ggplot() +
     #plot map
@@ -202,11 +308,15 @@ i.map <- function(id, homeCountry = NULL,
              fill = country.col,
              size = country.border.tick) +
 
+    #map range
+    xlim(ext[1],ext[2])+
+    ylim(ext[3],ext[4])+
+
     #plot line
-    geom_curve(data = countries, aes(x = homeCountry$x, #jitter to avoid problems with identical points (if any)
+    geom_curve(data = countries, aes(x = homeCountry$x,
                                      y = homeCountry$y,
-                                     xend = jitter(x,0.000001),
-                                     yend = jitter(y,0.000001)),
+                                     xend = jitter(x, 0.000001), #jitter to avoid problems with identical points (if any)
+                                     yend = jitter(y, 0.000001)), #jitter to avoid problems with identical points (if any)
                curvature = line.curvature,
                size = line.size,
                alpha = line.alpha,
@@ -242,14 +352,16 @@ i.map <- function(id, homeCountry = NULL,
       panel.border = element_blank(),
       panel.grid.major = element_blank(),
       panel.grid.minor = element_blank(),
+      plot.margin = unit(c(1,1,1,1), 'cm'),
       plot.background = element_rect(fill = sea.col, colour = sea.col))
 
   return(net)
 }
 
-#' Matrix matching country names and coordinates.
+#' Matrix matching country names, coordinates and GDP.
 #'
-#' A dataset that links author countries with the map using the coordinates.
+#' A dataset that links author countries with the map using the coordinates and with GDP per capita.
+#' Current GDP values are for 2020 (World Bank data: https://data.worldbank.org/indicator/NY.GDP.PCAP.PP.CD )
 #'
 #' @docType data
 #' @keywords datasets
